unit I2C;

interface

procedure Open_I2C_bus(com_port : PAnsiChar);
procedure Close_I2C_bus;
procedure I2C_wait;
procedure I2C_start;
procedure I2C_stop;
function I2C_tx_byte(b : byte) : boolean;
function I2C_rx_byte(acknowledge : boolean) : byte;

implementation

uses
  Windows;

var
  serial_port : dword;

procedure Open_I2C_bus(com_port : PAnsiChar);
begin
  serial_port := CreateFile(
    com_port,
    GENERIC_READ or GENERIC_WRITE,
    0,
    0,
    OPEN_EXISTING,
    FILE_ATTRIBUTE_NORMAL,
    0
  );
  EscapeCommFunction(serial_port, SETRTS);
  EscapeCommFunction(serial_port, SETDTR);
end;

procedure Close_I2C_bus;
begin
  CloseHandle(serial_port);
end;

procedure I2C_wait;
begin
  //void , insert here wait code for very fast systems
  //actual measured speed is 1,5 kbps on a P90...
end;

procedure SDA_output;
begin
  //there is no real difference between SDA_output
  //and SDA_input, because we have two unidirectional
  //lines instead of a single bidirectional one
  //This call is left here for compatibility with
  //other hardware implementations
  EscapeCommFunction(serial_port, SETDTR);
end;

procedure SDA_input;
begin
  //since DATA is open-collector, putting DTR high sets it as PC input
  EscapeCommFunction(serial_port, SETDTR);
end;

procedure SDA_high;
begin
  EscapeCommFunction(serial_port, SETDTR);
  I2C_wait;
end;

procedure SDA_low;
begin
  EscapeCommFunction(serial_port, CLRDTR);
  I2C_wait;
end;

procedure SCL_high;
begin
  EscapeCommFunction(serial_port, SETRTS);
  I2C_wait;
end;

procedure SCL_low;
begin
  EscapeCommFunction(serial_port, CLRRTS);
  I2C_wait;
end;

function SDA_value : boolean;
var
  stat : DWORD;
begin
  GetCommModemStatus(serial_port, stat);
  SDA_value := stat and MS_CTS_ON <> 0;
end;

procedure I2C_start;
begin
  SDA_output;
  SDA_high;
  SCL_high;
  SDA_low;
end;

procedure I2C_stop;
begin
  SDA_output;
  SDA_low;
  SCL_high;
  SDA_high;
end;

procedure I2C_tx_bit_1;
begin
  SCL_low;
  SDA_output;
  SDA_high;
  SCL_high;
  SCL_low;
end;

procedure I2C_tx_bit_0;
begin
  SCL_low;
  SDA_output;
  SDA_low;
  SCL_high;
  SCL_low;
end;

function I2C_rx_bit : boolean;
begin
  SDA_input;
  SCL_low;
  SCL_high;
  I2C_rx_bit := SDA_value;
  SCL_low;
end;

function I2C_tx_byte(b : byte) : boolean;
var
  i : byte;
begin
  for i := 0 to 7 do begin
    if b and $80 <> 0 then
      I2C_tx_bit_1
    else
      I2C_tx_bit_0;
    b := b shl 1;
  end;
  I2C_tx_byte := I2C_rx_bit;
end;

function I2C_rx_byte(acknowledge : boolean) : byte;
var
  i,
  retval : byte;
begin
  for i := 0 to 7 do begin
    retval := retval shl 1;
    If I2C_rx_bit then
      retval := retval + 1;
  end;
  if acknowledge Then
    I2C_tx_bit_0
  else
    I2C_tx_bit_1;
  I2C_rx_byte := retval;
end;

end.
